package com.gem.media.ad;

public interface AdStatusCallBack {

    void onAdLoad(String type, String ad_type, String reason, long time, String scenario);
    void onAdLoaded(String type, String ad_type, String reason, long time, String scenario);
    void onNoAdError(String type, String ad_type, String reason, long time, String scenario);
    void onAdShow(String type, String ad_type, String reason, long time, String scenario);
    void onAdClick(String type, String ad_type, String reason, long time, String scenario);
    void onAdDismiss(String type, String ad_type, String reason, long time, String scenario);
    void onAdLoadTimeout(String type, String ad_type, String reason, long time, String scenario);
}

